@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 #Exit if any undefined variable is used.
 set -u
@@ -28,15 +28,17 @@ HELP() {
   echo
   echo "Run build tests"
   echo "Options:"
-  echo " -k, --no-krazy        Don't run any Krazy tests"
-  echo " -c, --no-cppcheck     Don't run any cppcheck tests"
-  echo " -t, --no-tidy         Don't run any clang-tidy tests"
-  echo " -b, --no-scan         Don't run any scan-build tests"
-  echo " -s, --no-splint       Don't run any splint tests"
-  echo " -l, --no-clang-build  Don't run any clang-build tests"
-  echo " -g, --no-gcc-build    Don't run any gcc-build tests"
-  echo " -a, --no-asan-build   Don't run any ASAN-build tests"
-  echo " -d, --no-tsan-build   Don't run any TSAN-build tests"
+  echo " -m, --no-cmake-compat  Don't require CMake version compatibility"
+  echo " -k, --no-krazy         Don't run any Krazy tests"
+  echo " -c, --no-cppcheck      Don't run any cppcheck tests"
+  echo " -t, --no-tidy          Don't run any clang-tidy tests"
+  echo " -b, --no-scan          Don't run any scan-build tests"
+  echo " -s, --no-splint        Don't run any splint tests"
+  echo " -n, --no-ninja         Don't run any build tests with ninja"
+  echo " -l, --no-clang-build   Don't run any clang-build tests"
+  echo " -g, --no-gcc-build     Don't run any gcc-build tests"
+  echo " -a, --no-asan-build    Don't run any ASAN-build tests"
+  echo " -d, --no-tsan-build    Don't run any TSAN-build tests"
   echo
 }
 
@@ -44,7 +46,11 @@ COMMAND_EXISTS () {
     command -v $1 >/dev/null 2>&1
     if ( test $? != 0 )
     then
-    echo "$1 is not in your PATH. Either install this program or skip the assocatied test"
+    echo "$1 is not in your PATH. Either install this program or skip the associated test"
+    if ( test "$2" )
+    then
+      echo "or disable this check by passing the $2 command-line option"
+    fi
     exit 1
   fi
 }
@@ -85,7 +91,12 @@ CHECK_WARNINGS() {
   then
     echo "EXITING. $w warnings encountered"
     echo
-    cat $1 | grep "$2" | grep -v "$3" | sort | uniq
+    if ( test -n "$3")
+    then
+      cat $1 | grep "$2" | grep -v "$3" | sort | uniq
+    else
+      cat $1 | grep "$2" | sort | uniq
+    fi
     exit 1
   fi
 }
@@ -94,7 +105,7 @@ CHECK_WARNINGS() {
 # print warnings found in the compile-stage output
 # $1 = file with the compile-stage output
 COMPILE_WARNINGS() {
-  whitelist='\(no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|U_PLATFORM_HAS_WINUWP_API\|DB_DBM_HSEARCH\|const[[:space:]]DBT\|db\.h\)'
+  whitelist='\(i-cal-object\.c\|libical-glib-scan\.c\|no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|/gobject/gtype\.h\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|U_PLATFORM_HAS_WINUWP_API\|const[[:space:]]DBT\|-Llib\)'
   CHECK_WARNINGS $1 "warning:" "$whitelist"
 }
 
@@ -109,7 +120,8 @@ CPPCHECK_WARNINGS() {
 # print warnings find in the clang-tidy output
 # $1 = file with the clang-tidy output
 TIDY_WARNINGS() {
-  whitelist='\(Value[[:space:]]descriptions\|unused[[:space:]]declarations\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|modernize-\|cppcoreguidelines-pro-type-const-cast\|cppcoreguidelines-pro-type-reinterpret-cast\|cppcoreguidelines-owning-memory\|fuchsia.*\|hicpp-use-auto\|hicpp-no-malloc\|hicpp-use-nullptr\|hicpp-exception-baseclass\|hicpp-vararg\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-bounds-pointer-arithmetic\|google-build-using-namespace\|llvm-include-order\|hicpp-use-equals-default\|cppcoreguidelines-no-malloc\|g_type_class_add_private.*is[[:space:]]deprecated\)'
+  #whitelist='\(Value[[:space:]]descriptions\|unused[[:space:]]declarations\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|modernize-\|cppcoreguidelines-pro-type-const-cast\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-type-reinterpret-cast\|cppcoreguidelines-owning-memory\|fuchsia.*\|hicpp-use-auto\|hicpp-no-malloc\|hicpp-use-nullptr\|hicpp-exception-baseclass\|hicpp-vararg\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-bounds-pointer-arithmetic\|google-build-using-namespace\|llvm-include-order\|hicpp-use-equals-default\|cppcoreguidelines-no-malloc\|g_type_class_add_private.*is[[:space:]]deprecated\)'
+  whitelist='\(no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\)'
   CHECK_WARNINGS $1 "warning:" "$whitelist"
 }
 
@@ -117,8 +129,8 @@ TIDY_WARNINGS() {
 # print warnings found in the scan-build output
 # $1 = file with the scan-build output
 SCAN_WARNINGS() {
-  whitelist='\(no[[:space:]]link[[:space:]]for:\|g_type_class_add_private.*is[[:space:]]deprecated\|/vcc\.c\|/vobject\.c\|/icalsslexer\.c\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|icalerror.*Dereference[[:space:]]of[[:space:]]null[[:space:]]pointer\)'
-  CHECK_WARNINGS $1 "warning:" "$whitelist"
+  whitelist='\(no[[:space:]]link[[:space:]]for:\|g_type_class_add_private.*is[[:space:]]deprecated\|libical-glib-scan\.c\|/i-cal-object\.c\|/vcc\.c\|/vobject\.c\|/icalsslexer\.c\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|icalerror.*Dereference[[:space:]]of[[:space:]]null[[:space:]]pointer\|G_ADD_PRIVATE\)'
+  CHECK_WARNINGS $1 "warning:" $whitelist
 }
 
 #function CONFIGURE:
@@ -130,7 +142,12 @@ CONFIGURE() {
   mkdir -p $BDIR
   cd $BDIR
   rm -rf *
-  cmake .. $2 || exit 1
+  if ( test `echo $2 | grep -ci Ninja` -gt 0 )
+  then
+    cmake .. $2 || exit 1
+  else
+    cmake -G "Unix Makefiles" .. $2 || exit 1
+  fi
 }
 
 #function CLEAN:
@@ -147,9 +164,21 @@ CLEAN() {
 BUILD() {
   cd $TOP
   CONFIGURE "$1" "$2"
-  make 2>&1 | tee make.out || exit 1
+  MAKE=make
+  if ( test `echo $2 | grep -ci Ninja` -gt 0 )
+  then
+    MAKE=ninja
+  fi
+  $MAKE 2>&1 | tee make.out || exit 1
   COMPILE_WARNINGS make.out
-  make test 2>&1 | tee make-test.out || exit 1
+
+  if (test "`uname -s`" = "Darwin")
+  then
+    export DYLD_LIBRARY_PATH=$BDIR/lib
+  else
+    export LD_LIBRARY_PATH=$BDIR/lib
+  fi
+  $MAKE test 2>&1 | tee make-test.out || exit 1
   CLEAN
 }
 
@@ -171,6 +200,24 @@ GCC_BUILD() {
   echo "===== END GCC BUILD: $1 ======"
 }
 
+#function NINJA_GCC_BUILD:
+# runs a build test using gcc using the Ninja cmake generator
+# $1 = the name of the test (which will have "-ninjagcc" appended to it)
+# $2 = CMake options
+NINJA_GCC_BUILD() {
+  name="$1-ninjagcc"
+  if ( test $runninja -ne 1 )
+  then
+    echo "===== NINJA_GCC BUILD TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
+    return
+  fi
+  COMMAND_EXISTS "gcc"
+  echo "===== START NINJA_GCC BUILD: $1 ======"
+  SET_GCC
+  BUILD "$name" "$2 -G Ninja"
+  echo "===== END NINJA_GCC BUILD: $1 ======"
+}
+
 #function CLANG_BUILD:
 # runs a build test using clang
 # $1 = the name of the test (which will have "-clang" appended to it)
@@ -234,7 +281,7 @@ CPPCHECK() {
     echo "===== CPPCHECK TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "cppcheck"
+  COMMAND_EXISTS "cppcheck" "-c"
   echo "===== START SETUP FOR CPPCHECK: $1 ======"
 
   #first build it
@@ -255,6 +302,7 @@ CPPCHECK() {
            -D size_t="unsigned long" \
            -D bswap32="" \
            -D PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP=0 \
+           -D MIN="" \
            -D _unused="(void)" \
            -D F_OK=0 \
            -D R_OK=0 \
@@ -272,6 +320,7 @@ CPPCHECK() {
       grep -v Net-ICal | \
       grep -v icalssyacc\.c  | \
       grep -v icalsslexer\.c | \
+      grep -v vcc\.c | \
       grep -v _cxx\. | tee cppcheck.out
   CPPCHECK_WARNINGS cppcheck.out
   rm -f cppcheck.out
@@ -290,7 +339,7 @@ SPLINT() {
     echo "===== SPLINT TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "splint"
+  COMMAND_EXISTS "splint" "-s"
   echo "===== START SETUP FOR SPLINT: $1 ======"
 
   #first build it
@@ -313,6 +362,7 @@ SPLINT() {
   files="$files $BDIR/src/libical/*.c $BDIR/src/libical/*.h"
 
   splint $files \
+       -badflag \
        -weak -warnposix \
        -modobserver -initallelements -redef \
        -linelen 1000 \
@@ -374,11 +424,11 @@ CLANGTIDY() {
     echo "===== CLANG-TIDY TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "clang-tidy"
+  COMMAND_EXISTS "clang-tidy" "-t"
   echo "===== START CLANG-TIDY: $1 ====="
   cd $TOP
   SET_CLANG
-  CONFIGURE "$1-tidy" "$2 -DCMAKE_CXX_CLANG_TIDY=clang-tidy;-checks=*"
+  CONFIGURE "$1-tidy" "$2 -DCMAKE_CXX_CLANG_TIDY=clang-tidy"
   cmake --build . 2>&1 | tee make-tidy.out || exit 1
   TIDY_WARNINGS make-tidy.out
   CLEAN
@@ -395,7 +445,7 @@ CLANGSCAN() {
     echo "===== SCAN-BUILD TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "scan-build"
+  COMMAND_EXISTS "scan-build" "-b"
   echo "===== START SCAN-BUILD: $1 ====="
   cd $TOP
 
@@ -420,7 +470,7 @@ KRAZY() {
     echo "===== KRAZY TEST DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "krazy2all"
+  COMMAND_EXISTS "krazy2all" "-k"
   echo "===== START KRAZY ====="
   cd $TOP
   krazy2all 2>&1 | tee krazy.out
@@ -436,16 +486,18 @@ KRAZY() {
 
 ##### END FUNCTIONS #####
 
-#TEMP=`getopt -o hkctbslgad --long help,no-krazy,no-cppcheck,no-tidy,no-scan,no-splint,no-clang-build,no-gcc-build,no-asan-build,no-tsan-build -- "$@"`
-TEMP=`getopt hkctbslgad $*`
+#TEMP=`getopt -o hmkctbsnlgad --long help,no-cmake-compat,no-krazy,no-cppcheck,no-tidy,no-scan,no-splint,no-ninja,no-clang-build,no-gcc-build,no-asan-build,no-tsan-build -- "$@"`
+TEMP=`getopt hmkctbsnlgad $*`
 if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi
 # Note the quotes around `$TEMP': they are essential!
 eval set -- "$TEMP"
 
+cmakecompat=1
 runkrazy=1
 runcppcheck=1
 runtidy=1
 runscan=1
+runninja=1
 runclangbuild=1
 rungccbuild=1
 runasanbuild=1
@@ -454,15 +506,17 @@ runsplint=1
 while true ; do
     case "$1" in
         -h|--help) HELP; exit 1;;
-        -k|--no-krazy)       runkrazy=0;      shift;;
-        -c|--no-cppcheck)    runcppcheck=0;   shift;;
-        -t|--no-tidy)        runtidy=0;       shift;;
-        -b|--no-scan)        runscan=0;       shift;;
-        -s|--no-splint)      runsplint=0;     shift;;
-        -l|--no-clang-build) runclangbuild=0; shift;;
-        -g|--no-gcc-build)   rungccbuild=0;   shift;;
-        -a|--no-asan-build)  runasanbuild=0;  shift;;
-        -d|--no-tsan-build)  runtsanbuild=0;  shift;;
+        -m|--no-cmake-compat) cmakecompat=0;   shift;;
+        -k|--no-krazy)        runkrazy=0;      shift;;
+        -c|--no-cppcheck)     runcppcheck=0;   shift;;
+        -t|--no-tidy)         runtidy=0;       shift;;
+        -b|--no-scan)         runscan=0;       shift;;
+        -s|--no-splint)       runsplint=0;     shift;;
+        -n|--no-ninja)        runninja=0;      shift;;
+        -l|--no-clang-build)  runclangbuild=0; shift;;
+        -g|--no-gcc-build)    rungccbuild=0;   shift;;
+        -a|--no-asan-build)   runasanbuild=0;  shift;;
+        -d|--no-tsan-build)   runtsanbuild=0;  shift;;
         --) shift; break;;
         *)  echo "Internal error!"; exit 1;;
     esac
@@ -475,6 +529,40 @@ cd ..
 TOP=`pwd`
 BDIR=""
 
+#use minimum cmake version unless the --no-cmake-compat option is specified
+if ( test $cmakecompat -eq 1 )
+then
+  if ( test ! -e $TOP/CMakeLists.txt )
+  then
+    echo "Unable to locate the project top-level CMakeLists.txt.  Fix me"
+    exit 1
+  fi
+  # read the min required CMake version from the top-level CMake file
+  minCMakeVers=`grep -i cmake_minimum_required $TOP/CMakeLists.txt | grep VERSION | sed 's/^.*VERSION\s*//' | cut -d. -f1-2 | sed 's/\s*).*$//' | awk '{print $NF}'`
+  # adjust PATH
+  X=`echo $minCMakeVers | cut -d. -f1`
+  Y=`echo $minCMakeVers | cut -d. -f2`
+  if ( test -z $X -o -z $Y )
+  then
+    echo "Bad CMake version encountered in the $TOP/CMakeLists.txt"
+    exit 1
+  fi
+  Z=`echo $minCMakeVers | cut -d. -f3`
+  if ( test -z $Z )
+  then
+    minCMakeVers="$minCMakeVers.0"
+  fi
+  export PATH=/usr/local/opt/cmake-$minCMakeVers/bin:$PATH
+  # check the version
+  if ( test `cmake --version | head -1 | grep -c $minCMakeVers` -ne 1 )
+  then
+    echo "Not using cmake version $minCMakeVers"
+    echo "Maybe you need to install it into /usr/local/opt/cmake-$minCMakeVers (or use the -m option)"
+    exit 1
+  fi
+fi
+
+DEFCMAKEOPTS="-DCMAKE_BUILD_TYPE=Debug"
 CMAKEOPTS="-DCMAKE_BUILD_TYPE=Debug -DGOBJECT_INTROSPECTION=False -DICAL_GLIB=False -DICAL_BUILD_DOCS=False"
 UUCCMAKEOPTS="$CMAKEOPTS -DCMAKE_DISABLE_FIND_PACKAGE_ICU=True"
 TZCMAKEOPTS="$CMAKEOPTS -DUSE_BUILTIN_TZDATA=True"
@@ -492,7 +580,7 @@ CLANGTIDY test2 "$CMAKEOPTS"
 CLANGTIDY test2builtin "$TZCMAKEOPTS"
 
 #GCC based build tests
-GCC_BUILD testgcc1 ""
+GCC_BUILD testgcc1 "$DEFCMAKEOPTS"
 GCC_BUILD testgcc2 "$CMAKEOPTS"
 GCC_BUILD testgcc3 "$UUCCMAKEOPTS"
 GCC_BUILD testgcc4glib "$GLIBOPTS"
@@ -505,11 +593,20 @@ fi
 GCC_BUILD testgcc1builtin "-DUSE_BUILTIN_TZDATA=True"
 GCC_BUILD testgcc2builtin "$TZCMAKEOPTS"
 
-#Clang based build tests
-CLANG_BUILD testclang1 ""
+#Ninja build tests
+NINJA_GCC_BUILD testninjagcc1 "$DEFCMAKEOPTS"
+NINJA_GCC_BUILD testninjagcc2 "-DICAL_GLIB=True"
+NINJA_GCC_BUILD testninjagcc3 "-DICAL_GLIB=True -DICAL_GLIB_VAPI=ON -DGOBJECT_INTROSPECTION=True"
+NINJA_GCC_BUILD testninjagcc4 "-DSHARED_ONLY=True -DICAL_GLIB=False"
+NINJA_GCC_BUILD testninjagcc5 "-DSHARED_ONLY=True -DICAL_GLIB=True"
+NINJA_GCC_BUILD testninjagcc6 "-DSTATIC_ONLY=True -DICAL_GLIB=False"
+NINJA_GCC_BUILD testninjagcc7 "-DSTATIC_ONLY=True -DICAL_GLIB=True -DENABLE_GTK_DOC=False"
+NINJA_GCC_BUILD testninjagcc9 "-DSHARED_ONLY=True -DICAL_GLIB=True -DGOBJECT_INTROSPECTION=True -DICAL_GLIB_VAPI=ON"
+
+CLANG_BUILD testclang1 "$DEFCMAKEOPTS"
 CLANG_BUILD testclang2 "$CMAKEOPTS"
 CLANG_BUILD testclang3 "$UUCCMAKEOPTS"
-#broken with clang7 on Fedora29 CLANG_BUILD testclang4glib "$GLIBOPTS"
+CLANG_BUILD testclang4glib "$GLIBOPTS"
 if (test "`uname -s`" = "Linux")
 then
     echo "Temporarily disable cross-compile tests"
@@ -518,14 +615,14 @@ then
 fi
 
 #Address sanitizer
-ASAN_BUILD test1asan ""
+ASAN_BUILD test1asan "$DEFCMAKEOPTS"
 ASAN_BUILD test2asan "$CMAKEOPTS"
 ASAN_BUILD test3asan "$TZCMAKEOPTS"
 ASAN_BUILD test4asan "$UUCCMAKEOPTS"
 ASAN_BUILD test5asan "$GLIBOPTS"
 
 #Thread sanitizer
-TSAN_BUILD test1tsan ""
+TSAN_BUILD test1tsan "$DEFCMAKEOPTS"
 TSAN_BUILD test2tsan "$CMAKEOPTS"
 TSAN_BUILD test3tsan "$TZCMAKEOPTS"
 TSAN_BUILD test4tsan "$UUCCMAKEOPTS"
